@@ -1,6 +1,3 @@
-#' @import utils digest
-NULL
-
 # Like base::paste, but converts all string args to UTF-8 first.
 paste8 <- function(..., sep = " ", collapse = NULL) {
   args <- c(
@@ -52,13 +49,9 @@ registerMethods <- function(methods) {
     # c(package, genname, class)
     c("knitr", "knit_print", "html"),
     c("knitr", "knit_print", "shiny.tag"),
-    c("knitr", "knit_print", "shiny.tag.list")
+    c("knitr", "knit_print", "shiny.tag.list"),
+    c("knitr", "knit_print", "html_dependency")
   ))
-
-  # TODO: After rlang >= 0.4.12 hits CRAN, remove this and replace
-  # with ` #' @importFrom rlang obj_address`
-  # (lionel says rlang:::sexp_address() will be available for the next few years)
-  assign("obj_address", getFromNamespace("sexp_address", "rlang"), environment(.onLoad))
 }
 
 depListToNamedDepList <- function(dependencies) {
@@ -675,7 +668,6 @@ NULL
 #' @format NULL
 #' @docType NULL
 #' @keywords NULL
-#' @import rlang
 #' @export
 tags <- lapply(known_tags, function(tagname) {
   # Overwrite the body with the `tagname` value injected into the body
@@ -1717,6 +1709,9 @@ knit_print.html <- function(x, ..., inline = FALSE) {
 #' @export
 knit_print.shiny.tag.list <- knit_print.shiny.tag
 
+#' @rdname knitr_methods
+#' @export
+knit_print.html_dependency <- knit_print.shiny.tag
 
 #' Include Content From a File
 #'
@@ -1736,9 +1731,44 @@ knit_print.shiny.tag.list <- knit_print.shiny.tag
 #' @export
 includeHTML <- function(path) {
   lines <- readLines(path, warn=FALSE, encoding='UTF-8')
+
+  if (detect_html_document(lines)) {
+    rlang::warn(c(
+      "`includeHTML()` was provided a `path` that appears to be a complete HTML document.",
+      "x" = paste("Path:", path),
+      "i" = paste(
+        "Use `tags$iframe()` to include an HTML document.",
+        "You can either ensure `path` is accessible in your app or document",
+        "(see e.g. `shiny::addResourcePath()`) and pass the relative path to",
+        "the `src` argument. Or you can read the contents of `path` and pass",
+        "the contents to `srcdoc`."
+      )
+    ))
+  }
+
   return(HTML(paste8(lines, collapse='\n')))
 }
 
+detect_html_document <- function(lines) {
+  if (length(lines) > 1) {
+    lines <- paste8(lines, collapse = "\n")
+  }
+  lines <- trimws(lines)
+
+  # A complete html document starts with doctype declaration or opening <html>
+  if (!grepl("^<!DOCTYPE html>|<html", lines, ignore.case = TRUE)) {
+    return(FALSE)
+  }
+  # and ends by closing the `</html>` tag
+  if (!grepl("</html>$", lines, ignore.case = TRUE)) {
+    return(FALSE)
+  }
+
+  # There are more requirements for the HTML document to be technically complete
+  # and valid, but the above conditions are sufficient for us to warn that the
+  # document should not be treated as an HTML fragment.
+  TRUE
+}
 #' @note `includeText` escapes its contents, but does no other processing.
 #'   This means that hard breaks and multiple spaces will be rendered as they
 #'   usually are in HTML: as a single space character. If you are looking for
@@ -1757,11 +1787,11 @@ includeText <- function(path) {
 #' @rdname include
 #' @export
 includeMarkdown <- function(path) {
-  # markdown > v1.2 has removed the fragment.only argument
-  html <- if ("fragment.only" %in% names(formals(markdown::markdownToHTML))) {
+  # markdown >= v1.3 switched from markdownToHTML() to mark()
+  html <- if (packageVersion("markdown") < "1.3") {
     markdown::markdownToHTML(path, fragment.only = TRUE)
   } else {
-    markdown::markdownToHTML(path, options = '-standalone')
+    markdown::mark(path, output = NULL)
   }
   Encoding(html) <- 'UTF-8'
   return(HTML(html))
@@ -1917,8 +1947,10 @@ css <- function(..., collapse_ = "") {
     return(NULL)
   }
 
-  # Replace all '.' and '_' in property names to '-'
-  names(props) <- gsub("[._]", "-", tolower(gsub("([A-Z])", "-\\1", names(props))))
+  # Translate camelCase, snake_case, and dot.case to kebab-case
+  # For standard CSS properties only, not CSS variables
+  is_css_var <- grepl("^--", names(props))
+  names(props)[!is_css_var] <- standardize_property_names(names(props)[!is_css_var])
 
   # Create "!important" suffix for each property whose name ends with !, then
   # remove the ! from the property name
@@ -1931,3 +1963,11 @@ css <- function(..., collapse_ = "") {
 empty <- function(x) {
   length(x) == 0 || (is.character(x) && !any(nzchar(x)))
 }
+
+standardize_property_names <- function(x) {
+  # camelCase to kebab-case
+  x <- gsub("([A-Z])", "-\\1", x)
+  x <- tolower(x)
+  # snake_case and dot.case to kebab-case
+  gsub("[._]", "-", x)
+}
